#include <stdarg.h>
#include <signal.h>
#include <stdint.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mount.h>
#include <dirent.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <stdint.h>
#include "crc32.h"

#define SIZEERROR 22
#define SIZEZERO 20
#define WRITEERROR 40
#define READERROR 50
#define OPENERROR 60
#define MEMERROR 70
#define LENGTHERROR 80
#define CONTENTERROR 90
#define CRCERROR 30
#define HPERROR 46
//test -d dir -a ro/rw/wo -s x-yM/K 
#define MODE_RO 1
#define MODE_RW 2
#define MODE_WO 3
#define MODE_WS 4
#define MODE_RC 5
#define MODE_WD 6
#define MODE_WF 7

#define WRITEPAGE 0x8000

char testmode[10][32]={"NULL","RO","RW","WO","WS","RC","WD","WF"};

char testdir[512];
char logfile[512];
char errorfile[512];
char logpath[512];
char filepath[512];
char renamepath[512];

int mode=0;
unsigned int mins=0;
unsigned int maxs=0;
int needrandom;
unsigned int createfs=0;
unsigned int createneed=0;
int totalfnumber=0;
int zeronumber=0;
int totalok;
int totalfail;
int chunksize=0;
int partnum=0;
int timestamp = 0;
int loopcount = 0;
int blkcrc_check=0;
int ignore_random_str=0;
int msdelay=0;
struct ffmt{

	unsigned int crc;
	unsigned int intfn;
	unsigned int blocks;
	unsigned int magic;

};

int islog=0;
char *pwholefile=NULL;
int isbigmem=0;
extern uint32_t  crc32(uint32_t crc, char *buf, uint32_t len);

void get_random_bytes(void* buf, int nbytes);
void Sleep(int iSec,int iUsec);
void show_usage()
{
	printf("Usage:\n");	
	printf("-d : set the test directory, the test files will be write in this dir\n");
	printf("-t : specify the total size you want to write on disk. the program will stop till the total SIZE\n");
	printf("     is bigger than this value\n");
	printf("     example: -t <size>[g/m/G/M]\n");
	printf("-a : set mode. [ro/rc/wo/ws/wd] ro:readonly, it will check crc of the files and dump the error contents\n");
	printf("                                   for the files in the dir which -d option specified. \n");
	printf("                                rc:compare to ro, it will not dump the error contents\n");
	printf("                                ws:create files with O_SYNC, you need use -s at same time\n");
	printf("                                wd:create files with O_DIRECT, you need use -s at same time\n");
	printf("                                wo:create files with O_RDWR, you need use -s at same time\n");
	printf("-c : set the chunk size, when read/write file, the program will use fixed ramsize to test\n");

}

int getmode(char *mod)
{
	if(mod)
	{
		if(!strcasecmp(mod,"ro"))
			return MODE_RO;
		else if(!strcasecmp(mod,"rc"))
			return MODE_RC;
		else if(!strcasecmp(mod,"rw"))
			return MODE_RW;
		else if(!strcasecmp(mod,"wo"))
			return MODE_WO;
		else if(!strcasecmp(mod,"ws"))
			return MODE_WS;
		else if(!strcasecmp(mod,"wd"))
			return MODE_WD;
		else if(!strcasecmp(mod,"wf"))
			return MODE_WF;

	}
	return 0;
}

/* transfer string to number */
unsigned int simple_strtoui(const char *cp,char **endp,unsigned int base)
{
	unsigned int result = 0,value;

	if (*cp == '0') {
		cp++;
		if ((*cp == 'x') && isxdigit(cp[1])) {
			base = 16;
			cp++;
		}
		if (!base) {
			base = 8;
		}
	}
	if (!base) {
		base = 10;
	}
	while (isxdigit(*cp) && (value = isdigit(*cp) ? *cp-'0' : (islower(*cp)
					? toupper(*cp) : *cp)-'A'+10) < base) {
		result = result*base + value;
		cp++;
	}
	if (endp)
		*endp = (char *)cp;
	return result;
}



void p2log(char *format, ...)
{
	va_list args;
	FILE *fp;

	fp = fopen(logfile, "a+");
	if (!fp)
	{
		return ;
	}
	va_start(args, format);
	vfprintf(fp, format, args);
	va_end(args);
	fflush(fp);
	fclose(fp);
}

void dumpData(unsigned char *p, int size)
{
	//#define OPENDUMP
#ifdef OPENDUMP
	int i, c;

	printf ("hexdump %d bytes\n", size);
	while (size > 0) {
		printf ("\%08x:", (unsigned int)p);

		for (i = 0; i < 16; i ++) {
			if (i < size) {
				printf ("%02x ", p[i] & 0xFF);
			} else {
				printf ("   ");
			}
			if (i == 7) {
				printf (" ");
			}
		}

		printf (" |");

		for (i = 0; i < 16; i ++) {
			if (i < size) {
				c = p[i] & 0xFF;
				if ((c < 0x20) || (c >= 0x7F)) c = '.';
			} else {
				c = '\20';
			}
			printf ("%c", c);
		}
		printf ("|\n");

		size -= 16;
		p += 16;
	}
#endif
}


/* execute shell command */
#define SYSTEM_BUF_SIZE 0x200
int SYSTEM(const char *format, ...)
{
	char buf[SYSTEM_BUF_SIZE]="";
	va_list arg;

	va_start(arg, format);
	vsnprintf(buf,SYSTEM_BUF_SIZE, format, arg);
	va_end(arg);


	int ret = system(buf);

	usleep(1);
	return ret;
}

void badfile()
{
	printf("\n\n\n");
	printf("=======  =======  =======     =\n");
	printf("=        =      = =      =   ===\n");
	printf("=        =  ====  =  ====    ===\n");
	printf("=====    =  =     =  =       ===\n");
	printf("=        =   =    =   =       =\n");
	printf("=        =    =   =    =    \n");
	printf("======== =      = =      =    o\n");
	printf("\n");
	fflush(stdout);

}

void make_backup_fname(char *base)
{
	char random_string[16];
	bzero(random_string,sizeof(random_string));

	get_random_bytes(random_string, sizeof(random_string)-1);
	printf("====%s\n",random_string);
	random_string[sizeof(random_string)-1]='\0';
	/*prepare to backup this error file to logpath */
	bzero(errorfile,sizeof(errorfile));
	strcpy(errorfile,logpath);
	strcat(errorfile,"/err-");
	strcat(errorfile,(base+strlen(base)-10));
	strcat(errorfile,"-");
	strcat(errorfile,random_string);

}

int checkcrc_hp_detail(char *filen)
{
	int srcfd=-1;
	int rflags = O_RDONLY ;

	unsigned int dsize = 0;
	unsigned int fsize = 0;
	unsigned int numblocks = 0;
	unsigned int restbytes = 0;
	char *blockp = NULL;
	char cblknr[16];
	int blknr=9999;
	unsigned int blkcrc;
	int i;
	unsigned int lcrc,zcrc;
	int checkeveryblk=0;

	struct stat fstatus;
	char *tmpbuf=NULL;
	char *fn_num=NULL;
	char *fn_rnd=NULL;
	struct ffmt *newbuf;

	int lret=0;
	int readret=0;

	fn_num = filen+(strlen(filen)-8-10);
	fn_rnd = filen+(strlen(filen)-10);
	//printf("file number is :%s, rnd is:%s \n", fn_num, fn_rnd);

	srcfd=open(filen,rflags);
	if (srcfd < 0) {
		printf("open %s  fail\n",filen);
		return OPENERROR;
	}


	fstat(srcfd,&fstatus);
	fsize=fstatus.st_size;
//ignore zero length files
	if(fsize == 0)
	{
		if(islog)
			p2log("\n%s zero length\n",filen);
		//fflush(stdout);

		zeronumber++;
		close(srcfd);
		return 0;

	}

	tmpbuf=malloc(fsize);

	if(!tmpbuf)
	{
		printf("malloc error:0x%x\n",dsize);
		lret=MEMERROR;
		goto error;
	}
	memset(tmpbuf, 0 , fsize);
	zcrc = crc32( 0, (char *)tmpbuf, fsize);

	readret=read(srcfd, tmpbuf, fsize);
	if(readret != fsize)
	{
		printf("read file fail\n");
		if(islog)
			p2log("read file fail\n");
		lret=READERROR;
		goto error;

	}


	newbuf=(struct ffmt *)(tmpbuf + fsize - sizeof(struct ffmt));

	//dumpData(tmpbuf,512);
	/* check the detail of the file contents */
	if( newbuf->magic == 0xdeadbeef )
	{ 
		numblocks = newbuf->blocks;
		if (blkcrc_check == 1)
			printf("check the honey pot file (%d)blocks ... \n",newbuf->blocks);

		lcrc = crc32( 0, (char *)tmpbuf, (fsize - sizeof(struct ffmt)));

		if(newbuf->crc != lcrc){
			badfile();
			printf("\nchecking %s CRC value , head crc:0x%08x, recal crc:0x%08x\n",
					filen,newbuf->crc,lcrc);	
			fflush(stdout);


			lret=CRCERROR;
			goto error;
		}
		goto error;

	}
	else
	{
		//check if this file's contents is all 0
		lcrc = crc32( 0, (char *)tmpbuf, fsize);
		if(zcrc == lcrc) {
			printf("==============file contents all zero=============\n");
			goto error;
		}

		numblocks = fsize/512;
		restbytes = fsize - numblocks * 512;
		printf("this partly file has %d blocks and %d rest bytes\n", numblocks, restbytes);
		//do every block check
		checkeveryblk = 1;
		//lret = CONTENTERROR;


	}

	if ((blkcrc_check == 1) || (checkeveryblk == 1))
	{
		fflush(stdout);
		blockp=tmpbuf;
		for(i=0; i < numblocks; i++)
		{
			blkcrc=*(unsigned int *)(blockp+512-4);
			lcrc=crc32( 0, (char *)blockp, 512-4);
			if(blkcrc != lcrc)
			{
				if(islog)
					p2log("block crc check error on block %d(%08x:%08x)\n", i, blkcrc, lcrc);	

				printf("block crc check fail on block %d(%08x:%08x) on offset %d\n", 
						i, blkcrc, lcrc, i*512);
				lret = HPERROR;
			}

			bzero(cblknr,sizeof(cblknr));
			/*because the position was changed*/
			strncpy(cblknr, blockp+36, 8);
			//strncpy(cblknr, blockp+26, 8);
			//printf("cblknr:%s, blockp-tmpbuf:%d+36\n", cblknr, blockp-tmpbuf);
			blknr=simple_strtoui(cblknr, NULL, 10);
			if(blknr != i)
			{
				printf("blknr expect %d but the value is %d\n", i , blknr);
				fflush(stdout);
				if(islog)
					p2log("block index error on block %d:%d\n", i, blknr);	
				lret = HPERROR;
			}

			if(strncmp(fn_num,blockp+15, 8) != 0)
			{
				blockp[23]='\0';
				char fn_num_tmp[10];
				strncpy(fn_num_tmp, fn_num, 8);
				fn_num_tmp[8]='\0';
				printf("blk filename number except  %s but the value is %s\n",
						fn_num_tmp , (blockp+15));
				lret = HPERROR;

			}

			if(ignore_random_str == 0)
			{
				if(strncmp(fn_rnd,blockp+52, 10) != 0)
				{
					blockp[62]='\0';
					printf("blk rnd except  %s but the value is %s\n", fn_rnd , (blockp+52));
					lret = HPERROR;

				}
			}

			blockp+=512;

		}

		//if( lret != HPERROR)
		//	lret = 0;

		if (blkcrc_check == 1)
		{
			printf("blocks' scan done(%d)\n", lret);
			fflush(stdout);
		}

	}

error:
	close(srcfd);
	if(tmpbuf)
	{
		free(tmpbuf);
		tmpbuf=NULL;
	}
	return lret;
}



char *cmd_system(const char* format, ...)
{
	char* result =NULL; 
	FILE *fpRead;

	char buf[128]="";
	va_list arg;

	va_start(arg, format);
	vsnprintf(buf,128, format, arg);
	va_end(arg);

	result = malloc(128);
	if(result == NULL)
	{
		printf("%d mem error\n",__LINE__);
		return NULL;
	}
	bzero(result,128);
	fpRead = popen(buf, "r");

	while(fgets(result,(128-1),fpRead)!=NULL);

	if(fpRead!=NULL)
		pclose(fpRead);
	return result;

}

int check_file_in_test_dir(char *path, int indent)
{
	struct dirent* ent = NULL;
	DIR *pDir;
	char dir[512];
	unsigned int count=0;
	struct stat statbuf;  
	int crcret=0;
	int oldcrcret=0;
	totalfnumber=0;
	totalok=0;
	totalfail=0;

	char *pcmdret=NULL;
	int lastfilen=0;
	int failfilen=0;

	//pcmdret=cmd_system("ls -l %s/hp00* | wc -l",path);
	if(pcmdret != NULL)
	{
		if(strstr(pcmdret, "No such"))
		{
			lastfilen=0;
		}
		else
			lastfilen=simple_strtoui(pcmdret,NULL,10);
	}
	else
		lastfilen=0;


	if( (pDir=opendir(path))==NULL )
	{
		printf(  "Cannot open directory:%s\n", path );
		return OPENERROR;
	}

	if(islog)
	{
		p2log("start check, last file:%08x\n", lastfilen);
	}
	else
		printf("start check, last file:%08x\n", lastfilen);


	while( (ent=readdir(pDir))!=NULL )
	{
		count++;
		snprintf( dir, 512,"%s/%s", path, ent->d_name );    
		lstat( dir, &statbuf);
		if( S_ISDIR(statbuf.st_mode) )
		{

			if(strcmp( ".",ent->d_name) == 0 || strcmp( "..",ent->d_name) == 0)
			{
				continue;
			}    
			/*
			   printf( "%*s subdir:%s/\n", indent, "", ent->d_name );
			   List( dir, indent+4 );        
			 */        
		}
		else
		{

			totalfnumber++;
			crcret=checkcrc_hp_detail(dir);
			if(crcret > 0)
			{
				if(CRCERROR == crcret)
					printf("\nFile %s checksum ERROR\n",dir);
				else if(SIZEERROR == crcret) 
					printf("\nFile %s size ERROR\n",dir);
				else if(CONTENTERROR == crcret)
					printf("\nFile %s CONTENT ERROR\n",dir);
				else
					printf("\nFile %s other ERROR(%d)\n",dir, crcret);

				fflush(stdout);

				oldcrcret=crcret;

				totalfail++;
				*(ent->d_name+10)='\0';
				failfilen=simple_strtoui((ent->d_name+2),NULL,16);
				if(failfilen != lastfilen)
				{
					printf("An error file %08x,BUT last file %08x\n",failfilen,lastfilen);
					if(islog)
						p2log("An error file %08x,BUT last file %08x\n",failfilen,lastfilen);

				}
				//return crcret;
			}   
			else
			{
				printf("\rFile %s checksum OK",dir);
				totalok++;
				fflush(stdout);
			}
			if(msdelay > 0)
			Sleep(0, msdelay*1000);//delay 1ms

		}        
	}
	closedir(pDir);	

	if(count<=2)
	{
		printf("NO file availabl ???\n");
		crcret=0;
		fflush(stdout);

	}
	printf("\n");
	if((oldcrcret == crcret))
	{
		return crcret;
	}
	else
		return oldcrcret;
}

static int statici=0;

unsigned int get_random(unsigned int a, unsigned int b)
{

	if(a < b)
	{
		srand((unsigned int)time((time_t *)NULL)+rand());
		return ((rand() % (b-a))+ a);
	}
	return 0;
}

unsigned int get_content()
{
	statici=statici+1;
	return statici;

}

void get_random_bytes(void* buf, int nbytes)
{
	int i;


	char str[64] = "00123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"; 

	srand((unsigned int)time((time_t *)NULL)+rand());
	for(i=0;i<nbytes;i++)
	{

		//sprintf(ss,"%c",str[(rand()%62)]);
		//strcat(buf,ss);
		*(char *)(buf+i)=str[(rand()%62)];

	}

	return;
}


void addChkSum(char *buffer,unsigned int bytes)
{

	unsigned int crc=0;
	crc=crc32( crc, (char *)buffer, bytes-4);	
	*((unsigned int*)(buffer+bytes-4))=crc;
}

#define SWAPWORD(crc) ( ((crc & 0x000000ff) << 24) | \
		(( (crc>>8)  & 0x000000ff) << 16) | \
		(( (crc>>16) & 0x000000ff) <<  8) | \
		(( (crc>>24) & 0x000000ff)      ) )

void Sleep(int iSec,int iUsec)
{
	struct timeval tv;
	tv.tv_sec=iSec;
	tv.tv_usec=iUsec;
	select(0,NULL,NULL,NULL,&tv);
}

int create_honey_file(int numFiles, int filesize_by_block , int pn)
{
	int i, fd, amount, p;
	int fsystemNum = pn;
	//int numFiles = 0;
	char block[512];
	char *pattern;
	char devicename[32];
	char filename[64];
	char fn[64];
	char rnd[16];

	unsigned int epoch_32=0;
	//	time_t tt;

	char dirname[512];
	char newname[512];
	char *devPattern = "mmcblk1p";
	char *filePattern = "hp";
	int numBlocks = filesize_by_block;
	int crc_overall=0;
	char progname[]="create_honey_file";

	int openflg=O_RDWR|O_CREAT;

	if(mode == MODE_WS){
		openflg |= O_SYNC;
		printf("flag:O_SYNC|O_RDWR|O_CREAT\n");
	}
	else if(mode == MODE_WD){
		openflg |= (O_DIRECT|O_SYNC);
		printf("flag:O_SYNC|O_RDWR|O_CREAT|O_DIRECT\n");   
	}
	else if(mode == MODE_WF)
		printf("flag: fsync\n");

	if(renamepath[0] != '0') {
		mode = MODE_WF;
		printf("transaction safe test, flag: fsync,O_RDWR|O_CREAT\n");
		openflg=O_RDWR|O_CREAT;
	}


	if (numFiles == 0){
		fprintf(stderr,"%s: you have to specify -t\n", progname);
		exit(1);
	}

	if (fsystemNum == 0 ){
		fprintf(stderr,"%s: you have to specify -p\n", progname);
		exit(1);
	}

	struct ffmt fh;

	printf("\n\n\n\n");
	for (i = 1; i<= numFiles; i++){
		int blocknr;

		//printf("\nCreate file : 0x%08x size: %04d*512",i,fh.blocks);
		bzero(&fh, sizeof(struct ffmt));
		if(pwholefile)
			bzero(pwholefile, (numBlocks+1)*512);

		fh.magic=0xdeadbeef;
		crc_overall=0;
		bzero(fn,sizeof(fn));
		bzero(rnd,sizeof(rnd));
		bzero(dirname, sizeof(dirname));

		if(ignore_random_str == 0)
		{
			get_random_bytes(rnd, 10);
			rnd[12]='\n';
			sprintf(fn, "/%s%08x%s", filePattern, i,rnd);
		}
		else
		{
			sprintf(fn, "/%s%08x", filePattern, i);
		}

		strcpy(dirname,testdir);
		strcat(dirname, fn);

		fd = open(dirname, O_RDWR|O_CREAT, 0600);

		if (-1==fd){
			fprintf(stderr,"%s: couldn't open('%s') - giving up: %s\n",
					progname, dirname, strerror(errno));

			if(islog)
				p2log("%s: couldn't open('%s') - giving up: %s\n",
						progname, dirname, strerror(errno));

			return 0;
		}

		snprintf(devicename, sizeof(devicename), "%s%d", devPattern, fsystemNum);
		snprintf(filename, sizeof(filename), "%s%08x", filePattern, i);

		//LFSR32_init(&crc_overall);

		if(timestamp)
		{
			//tt = time(NULL);
			epoch_32 = time(NULL);
			epoch_32 = SWAPWORD(epoch_32);
			//printf("epoch %08x\n", epoch_32);
		}


		for(blocknr = 0; blocknr<numBlocks; blocknr++)
		{
			memset(block, ' ', sizeof(block));
			/* text block identifier */
			strncpy(block + 0, devicename, strlen(devicename));
			block[12] = '|';
			strncpy(block + 13, filename, strlen(filename) );
			//snprintf(block + 25, 8, "|%05d|", blocknr);
			block[23] = '|';
			/*exchange the position of blocknr and loop count*/
			snprintf(block + 24, 11, "%08d|", loopcount);
			snprintf(block + 36, 10, "%08d|", blocknr);

			//snprintf(block + 25, 11, "|%08d|", blocknr);
			//snprintf(block + 36, 10, "%08d|", loopcount);

			/* create the 119 patterns */
			//pattern = &block[32];
			//if(timestamp)
			memcpy( &block[48], &epoch_32, sizeof(epoch_32));
			strncpy( block+52, rnd, 10);

			pattern = &block[64];
			for (p=0; p<111; p++){
				pattern[0] = (unsigned char) fsystemNum;
				pattern[1] = (unsigned char) i;
				pattern[2] = (unsigned char) ((blocknr >> 8) & 0xFF);
				pattern[3] = (unsigned char) (blocknr & 0xFF);

				pattern += 4;
			}
			addChkSum(block, sizeof(block) );
			if(isbigmem == 0)
			{
				amount = sizeof(block);
				if (amount != write(fd, block, amount)){
					fprintf(stderr,"%s: couldn't write('%s', ,%d) - giving up: %s\n",
							progname, dirname, amount, strerror(errno));
					if(islog)
						p2log("error %s: couldn't write('%s', ,%d) - giving up: %s\n",
								progname, dirname, amount, strerror(errno));

					return 0;
				}
			}
			else
				memcpy((pwholefile+sizeof(block)*blocknr), block,sizeof(block));

			fh.blocks++;
			fh.intfn=i;
			/* compute checksum over complete block */
			//LFSR32_process( &crc_overall , block , 0L , sizeof(block) );
			crc_overall=crc32( crc_overall, (char *)block, sizeof(block));	

		}
		//overallChksum = crc_overall;//LFSR32_get_state(&crc_overall);
		fh.crc=crc_overall;
		if(isbigmem == 0)
		{
			amount = sizeof(struct ffmt);
			/* write in host endian */
			if (amount != write(fd, &fh, amount)){
				fprintf(stderr,"%s: couldn't write('%s', ,%d) - giving up: %s\n",
						progname, dirname, amount, strerror(errno));
				if(islog)
					p2log("error %s: couldn't write('%s', ,%d) - giving up: %s\n",
							progname, dirname, amount, strerror(errno));

				return 0;
			}
		}
		else
		{
			memcpy((pwholefile+sizeof(block)*blocknr), (unsigned char *)&fh,sizeof(struct ffmt));
			amount = sizeof(struct ffmt) + numBlocks*512;
			/* write in host endian */
			if (amount != write(fd, pwholefile, amount)){
				fprintf(stderr,"%s: couldn't write('%s', ,%d) - giving up: %s\n",
						progname, dirname, amount, strerror(errno));
				if(islog)
					p2log("error %s: couldn't write('%s', ,%d) - giving up: %s\n",
							progname, dirname, amount, strerror(errno));

				return 0;
			}

		}

		if(mode == MODE_WF) {
			fsync(fd);
			//printf("do fsync\n");
		}
		close(fd);

		if(ignore_random_str == 1)
		{
			bzero(fn,sizeof(fn));
			bzero(newname, sizeof(newname));
			sprintf(fn, "/%s%08x_copy12345", filePattern, i);

			strcpy(newname,renamepath);
			strcat(newname, fn);
			rename(dirname, newname);
		}
		else
		{
			bzero(newname, sizeof(newname));
			strcpy(newname,renamepath);
			strcat(newname, fn);
			rename(dirname, newname);
			//printf("rename %s to %s\n", dirname, newname);
		}

	}

	printf("Create 0x%08x files DONE\n", i);
	//fflush(stdout);
	return 0;
}

int create_file_in_test_dir()
{

	int ret=create_honey_file(createneed, chunksize, partnum );
	return ret;
}

int main(int argc, char *argv[])
{
	int ret=0;
	int c=0;

	char tmp[64];
	bzero(tmp,64);
	bzero(testdir,sizeof(testdir));
	bzero(renamepath,sizeof(renamepath));
	bzero(logfile,sizeof(logfile));
	bzero(logpath,sizeof(logpath));
	printf("%s %s %s\n",__FILE__, __DATE__, __TIME__);	
	//long test=0x0000000012345678L;
	//uint32_t test2=test;
	//printf("char*:%d,int:%d,long:%d,void *:%d,test:%016p,%x\n", sizeof(char*), sizeof(int), sizeof(long), sizeof(void*),test,test2);

	for (;;) {
		c = getopt( argc, argv, "d:a:p:t:v:c:f:b:i:e:r:m:h");
		if (c == EOF) break;

		switch (c) {
			case 'd':
				strcpy(testdir,optarg);
				break;
			case 'r':
				strcpy(renamepath,optarg);
				break;

			case 'f':
				strcpy(testdir,optarg);
				blkcrc_check=1;
				break;
			case 'v':
				strcpy(logpath,optarg);
				strcpy(logfile,logpath);
				strcat(logfile,"/stable_test.log");
				islog=1;
				break;
			case 'a':
				mode=getmode(optarg);
				break;
			case 'p':
				partnum=simple_strtoui(optarg,NULL,10);
				break;
			case 'b':
				isbigmem=simple_strtoui(optarg,NULL,10);
				break;

			case 'c':
				chunksize=simple_strtoui(optarg,NULL,10);
				printf("block counter:%d(0x%x)\n",chunksize,chunksize);
				break;
			case 't':
				createneed=simple_strtoui(optarg,NULL,10);
				break;

			case 'i':
				loopcount=simple_strtoui(optarg,NULL,10);
				break;
			case 'x':
				if(*optarg == '1')
					ignore_random_str = 1;
				break;

			case 'e':
				timestamp=simple_strtoui(optarg,NULL,10);
				break;
			case 'm':
				msdelay=simple_strtoui(optarg,NULL,10);
				break;

			case 'h': 
			default:
				show_usage();
				exit(1);
		}
	}

	printf("delay mode:%d\n", msdelay);
	if(testdir[0]==0 || mode == 0  )
	{
		printf("forget option -d or -a\n");
		exit(1);
	}

	if(blkcrc_check)
	{
		printf("==\n");
		checkcrc_hp_detail(testdir);
		return 0;
	}

	if((mode == MODE_WO) || (mode == MODE_WS) || (mode == MODE_WD) || (mode == MODE_WF)){		
		printf("test dir:%s,test mode:%d|%s,createneed:%d\n",
				testdir,mode,testmode[mode],createneed);

		if(ignore_random_str == 1)
		{
			if(renamepath[0] == 0 )
			{
				printf("forget option -r \n");
				exit(1);
			}		
		}

		if(loopcount == 0)
		{
			printf("forget option -i\n");
			return 0;
		}

		if(chunksize == 0)
		{
			printf("error chunksize 0?\n");
			return 0;
		}		

		if(createneed == 0)
		{
			printf("error createneed 0?\n");
			return 0;
		}	
	}	


	switch(mode)
	{
		case MODE_RO:
		case MODE_RC:
			printf("File checksum check in test dir:%s, test mode:%d|%s\n",
					testdir,mode,testmode[mode]);

			ret=check_file_in_test_dir(testdir,0);

			printf("%d files checked\n",totalfnumber);
			printf("%d files is zero length,%d files checksum ok\n",zeronumber,totalok);
			printf("%d files check FAIL!!!\n",totalfail);
			if(islog)
			{
				p2log("%d files, ",totalfnumber);
				p2log("%d zero length, %d checksum ok\n",zeronumber,totalok);
				p2log("%d check FAIL!!!\n",totalfail);

				p2log("check end\n\n");
			}

			break;
		case MODE_WO:
		case MODE_WS:
		case MODE_WD:
		case MODE_WF:
			pwholefile=NULL;
			if(isbigmem)
			{
				pwholefile=malloc((chunksize+1)*512);
				if(pwholefile == NULL)
				{
					printf("error alloc mem\n");
					return 0;
				}
			}
			ret=create_file_in_test_dir();
			free(pwholefile);
			break;
		default:
			show_usage();
			break;
	}

	printf("\n\n");
	return ret;
}

